module IntMap=Map.Make(struct type t=int let compare=compare end)

type job={
  tileset:int IntMap.t;
  assembly:int IntMap.t;
  posX:int;
  posY:int
}
let sharable _=true
type result=(string*job) list

let timeout=1800.
let ping=600.

let wgl=5
let mgl=(1 lsl wgl)-1

let south x=(x lsr (wgl*4)) land mgl
let west x=(x lsr (wgl*3)) land mgl
let north x=(x lsr (wgl*2)) land mgl
let east x=(x lsr (wgl*1)) land mgl
let color x=x land mgl

let withS t x=(t land (lnot (mgl lsl (wgl*4)))) lor (x lsl (wgl*4))
let withW t x=(t land (lnot (mgl lsl (wgl*3)))) lor (x lsl (wgl*3))
let withN t x=(t land (lnot (mgl lsl (wgl*2)))) lor (x lsl (wgl*2))
let withE t x=(t land (lnot (mgl lsl wgl))) lor (x lsl wgl)
let withC t x=(t land (lnot (mgl))) lor x

let add_result a b c=(a,c)::b
(* val add_result:string->result->job->result *)
let depth j=IntMap.cardinal j.assembly


(* val depth:job->int *)
let signature_size=1 lsl 10
(* val signature_size:int *)

let colors n=
  let _=Random.init 2014 in
  let rec col i=
    if i=0 then Printf.sprintf "#%06x" (Random.int 0x1000000) else (
      let _=Random.int 0x1000000 in
      col (i-1)
    )
  in
  col n

let html_job buf j=
  Buffer.add_string buf
  "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" style=\"height:";

  let maxy=IntMap.fold (fun pos _ m->max m (pos land 0xffff)) j.assembly 0 in
  let c=10 in

  Buffer.add_string buf (Printf.sprintf "%dpx\">" ((maxy+1)*2*c));

  IntMap.iter (fun pos t->
    try
      let tile=IntMap.find t j.tileset in
      let x,y=pos lsr 16, pos land 0xffff in
      let x=x*2*c and y=(maxy-y+1)*2*c in

      Buffer.add_string buf "<polyline points=\"";
      Buffer.add_string buf (Printf.sprintf "%d,%d %d,%d %d,%d" x y x (y-2*c) (x+c) (y-c));
      Buffer.add_string buf "\" style=\"fill:";
      Buffer.add_string buf (colors (west tile));
      Buffer.add_string buf ";\"/>";

      Buffer.add_string buf "<polyline points=\"";
      Buffer.add_string buf (Printf.sprintf "%d,%d %d,%d %d,%d" x (y-2*c) (x+2*c) (y-2*c) (x+c) (y-c));
      Buffer.add_string buf "\" style=\"fill:";
      Buffer.add_string buf (colors (north tile));
      Buffer.add_string buf ";\"/>";

      Buffer.add_string buf "<polyline points=\"";
      Buffer.add_string buf (Printf.sprintf "%d,%d %d,%d %d,%d" (x+2*c) (y-2*c) (x+2*c) y (x+c) (y-c));
      Buffer.add_string buf "\" style=\"fill:";
      Buffer.add_string buf (colors (east tile));
      Buffer.add_string buf ";\"/>";

      Buffer.add_string buf "<polyline points=\"";
      Buffer.add_string buf (Printf.sprintf "%d,%d %d,%d %d,%d" x y (x+2*c) y (x+c) (y-c));
      Buffer.add_string buf "\" style=\"fill:";
      Buffer.add_string buf (colors (south tile));
      Buffer.add_string buf ";\"/>";

      Buffer.add_string buf (Printf.sprintf "<rect x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\""
                               x (y-2*c) (2*c) (2*c));
      Buffer.add_string buf " style=\"stroke:black;stroke-width:0.5;fill:none;\"/>";

    with
        _->()
  ) j.assembly;
  Buffer.add_string buf "</svg>"


let html_result buf r=
  List.iter (fun (host,x)->
    Buffer.add_string buf "<p>Host: ";
    Buffer.add_string buf host;
    Buffer.add_string buf "<br/><br/>";
    html_job buf x;
    Buffer.add_string buf "</p>";
  ) r
