
(*
This module starts by generating a valid initial state. This means generating
an initial tileset, with all colors different. Then, loading the key, which is
hardcoded into the program, in our case.
*)

module Pats=Parry_server.Server(Job)
open Parry_server
open Parry_common
open Job

let initial=
  let rec initial i m=
    if i>=13 then m else
      let t=withS 0 (2*i) in
      let t=withN t (2*i+1) in
      let t=withW t (2*i) in
      let t=withE t (2*i+1) in
      let t=withC t Job.mgl in
      initial (i+1) (IntMap.add i t m)
  in
  initial 0 IntMap.empty

(*
let initial=
  List.fold_left (fun m (a,b)->IntMap.add a b m)
    IntMap.empty
    [(0,withC (withW (withS (withE (withN 0 0) 1) 2) 1) 0);
     (1,withC (withW (withS (withE (withN 0 2) 1) 0) 1) 1);
     (2,withC (withW (withS (withE (withN 0 2) 9) 2) 11) 0);
     (3,withC (withW (withS (withE (withN 0 12) 11) 0) 9) 0);
     (4,withC (withW (withS (withE (withN 0 12) 1) 12) 1) 1);
     (5,withC (withW (withS (withE (withN 0 12) 11) 12) 11) 0);
     (6,withC (withW (withS (withE (withN 0 0) 11) 0) 11) 0);
     (7,withC (withW (withS (withE (withN 0 2) 29) 2) 29) 0);
     (8,withC (withW (withS (withE (withN 0 12) 29) 12) 29) 0);
     (9,withC (withW (withS (withE (withN 0 12) 37) 0) 29) 0);
     (10,withC (withW (withS (withE (withN 0 2) 37) 2) 37) 0);
     (11,withC (withW (withS (withE (withN 0 12) 11) 12) 9) 0);
     (12,withC (withW (withS (withE (withN 0 0) 29) 12) 37) 0)]
*)

let st=Pats.initial_state [{tileset=initial;
                            assembly=IntMap.empty;
                            posX=0;
                            posY=0
                           }] []
let key=
  let open Cryptokit.RSA in
  {Parry_common.blank_key with size=(String.length Key.n) lsl 3; n=Key.n;e=Key.e }

let _=
  Pats.add_key st key

(*i*)
let daemonize f=
  if Unix.fork ()=0 then (
    let _=Unix.setsid () in
    if Unix.fork ()=0 then (
      let devnull=Unix.openfile "/dev/null" [Unix.O_RDWR] 0o644 in
      Unix.dup2 devnull Unix.stdin;
      let std0=Unix.openfile "parry.stdout" [Unix.O_RDWR;Unix.O_CREAT;Unix.O_TRUNC] 0o644 in
      Unix.dup2 std0 Unix.stdout;
      let std1=Unix.openfile "parry.stderr" [Unix.O_RDWR;Unix.O_CREAT;Unix.O_TRUNC] 0o644 in
      Unix.dup2 std1 Unix.stderr;
      Sys.set_signal Sys.sighup Sys.Signal_ignore;
      f ()
    ) else
      exit 0
  ) else
    exit 0

let _=
  if Sys.file_exists "state" then (
    let o=open_in "state" in
    fst st:=input_value o;
    close_in o;
  ) else (
    let unlink f=if Sys.file_exists f then Unix.unlink f in
    unlink "parry.log";
    unlink "available.log";
    unlink "ongoing.log";
    unlink "min_available.log";
    unlink "min_ongoing.log";
  )

(*
After this, the actual server starts:
*)

let doit f=f ()
let _=
  (* daemonize *)
  doit
    (fun ()->

(*
Handlers are set for the {\tt term} and {\tt int} signals,
to dump the server's state before exiting.
*)
      let handler _=
        let (rst,mst)=st in
        let _=Thread.create (fun ()->
          Mutex.lock mst;
          let o=open_out "state.tmp" in
          output_value o !rst;
          close_out o;
          Unix.rename "state.tmp" "state";
          exit 0;
        ) ()
        in
        ()
      in
      Sys.set_signal Sys.sigterm (Sys.Signal_handle handler);
      Sys.set_signal Sys.sigint (Sys.Signal_handle handler);

(*
The first thread dumps the server's state into a file called ``{\tt state}''.
In order to make this operation atomic and avoid data corruption,
it uses another file called {\tt state.tmp}.
*)

      let _=Thread.create (fun ()->
        while true do
          Thread.delay 600.;
          let (rst,mst)=st in
          Mutex.lock mst;
          (try
             let o=open_out "state.tmp" in
             output_value o !rst;
             close_out o;
             Unix.rename "state.tmp" "state";
           with _->());
          Mutex.unlock mst;
        done
      ) ()
      in

(*
Then, a web interface is started in another thread:
*)

      let _=Thread.create (fun ()->Pats.webui 8000 st) () in

(*
Finally, the actual server, proven in Section \ref{sect:parry-server},
is started:
*)
      Pats.server st
    )
